package org.processmining.dataawarecnetminer.parallel;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;

import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

public interface TaskExecutor {

	public static final class Factory {

		private Factory() {
			super();
		}

		private final static TaskExecutor DEFAULT_PARTITIONED = new TaskExecutor() {

			@SuppressWarnings("unchecked")
			public <T> void execute(ExecutorService executor, Iterable<T> input, final Command<T> task)
					throws InterruptedException {
				Iterable<List<T>> partitions;
				if (input instanceof List) {
					partitions = Lists.partition((List<T>) input, 10000);
				} else {
					partitions = Iterables.partition(input, 10000);
				}

				if (partitions instanceof List && ((List<T>) partitions).size() == 1) {
					for (T element : partitions.iterator().next()) {
						task.execute(element);
					}
				} else {
					List<Callable<Void>> callables = new ArrayList<>();
					for (final List<T> partition : partitions) {
						callables.add(new Callable<Void>() {

							public Void call() throws Exception {
								for (T element : partition) {
									task.execute(element);
								}
								return null;
							}
						});
					}

					try {
						executor.invokeAll(callables);
					} catch (InterruptedException e) {
						Thread.currentThread().interrupt();
						throw e;
					}
				}

			}
		};

		public static TaskExecutor getInstance() {
			return DEFAULT_PARTITIONED;
		}

	}

	<T> void execute(ExecutorService executor, Iterable<T> input, Command<T> task) throws InterruptedException;

}
