package org.processmining.dataawarecnetminer.visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristicConfig;
import org.processmining.dataawarecnetminer.util.ExtendedDoubleSlider;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class CausalGraphConfigurationPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private final NiceDoubleSlider dependencySlider;
	private final NiceDoubleSlider l1Slider;
	private final NiceDoubleSlider l2Slider;
	private final NiceDoubleSlider relativeToBestSlider;
	private final NiceDoubleSlider frequencySlider;

	private final JCheckBox allConnectedCheckbox;
	private final JCheckBox acceptedConnectedCheckbox;

	public CausalGraphConfigurationPanel(final DependencyHeuristicConfig config) {
		super();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		frequencySlider = new ExtendedDoubleSlider("Frequency", 0.0d, 1.0d, config.getObservationThreshold(),
				Orientation.HORIZONTAL);
		frequencySlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					config.setObservationThreshold(frequencySlider.getValue());
				}
			}

		});
		dependencySlider = new ExtendedDoubleSlider("Dependency", 0.0d, 1.0d, config.getDependencyThreshold(),
				Orientation.HORIZONTAL);
		dependencySlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					config.setDependencyThreshold(dependencySlider.getValue());
				}
			}

		});
		l1Slider = new ExtendedDoubleSlider("L1 Loop", 0.0d, 1.0d, config.getL1Threshold(), Orientation.HORIZONTAL);
		l1Slider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					config.setL1Threshold(l1Slider.getValue());
				}
			}

		});
		l2Slider = new ExtendedDoubleSlider("L2 Loop", 0.0d, 1.0d, config.getL2Threshold(), Orientation.HORIZONTAL);
		l2Slider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					config.setL2Threshold(l2Slider.getValue());
				}
			}

		});
		relativeToBestSlider = new ExtendedDoubleSlider("Relative To Best", 0.0d, 1.0d,
				config.getRelativeToBestThreshold(), Orientation.HORIZONTAL);
		relativeToBestSlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					config.setRelativeToBestThreshold(relativeToBestSlider.getValue());
				}
			}

		});

		add(frequencySlider);
		add(Box.createVerticalGlue());
		add(dependencySlider);
		add(Box.createVerticalGlue());
		add(l1Slider);
		add(Box.createVerticalGlue());
		add(l2Slider);
		add(Box.createVerticalGlue());
		add(relativeToBestSlider);
		
		add(Box.createVerticalGlue());

		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.X_AXIS));
		bottomPanel.setBackground(null);
		bottomPanel.setOpaque(false);

		allConnectedCheckbox = SlickerFactory.instance().createCheckBox("All Tasks Connected",
				config.isAllTasksConnected());
		allConnectedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				config.setAllTasksConnected(allConnectedCheckbox.isSelected());
			}
		});
		bottomPanel.add(allConnectedCheckbox);
		acceptedConnectedCheckbox = SlickerFactory.instance().createCheckBox("Accepted Tasks Connected",
				config.isAcceptedTasksConnected());
		acceptedConnectedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				config.setAcceptedTasksConnected(acceptedConnectedCheckbox.isSelected());
			}
		});
		bottomPanel.add(acceptedConnectedCheckbox);

		add(bottomPanel);
	}

}